#ifndef NMRsim_logfile_h_
#define NMRsim_logfile_h_

/*! \file 
  \brief  Objects for working with log files */

#include "libcmatrix/matlabio.h"

//! class to control logging output
class logfile_controller {
public:
  logfile_controller(const char* fname, int flags); //!< open log file \a fname with flags \a flags
  ~logfile_controller();

  //! flags for log file
  enum { DOUBLE=1, //!< double precision output
	 MATLAB=2, //!< use Matlab format
	 ASCII=4, //!< use ASCII output
	 APPEND=8 //!< append to existing file
  };

  template<class T> void write(const Matrix<T>& a, const char* name) const; //! write matrix named \a name to log 
  template<class T> void write(const BlockedMatrix<T>& a, const char* name) const; //! write blocked matrix named \a name to log
  void write(const char*, const char* name) const; //!< write string to log
  void flush() const; //!< ensure log file stream is flushed

  static void parse_raw(char*&, int&); //!< \internal
  static void maketitle(char*, int, const char*); //!< \internal
 friend class smartptr<logfile_controller,false>; //!< hack to allow use of smartptr when constructor is private

  static ContextWarning<> ignoring_warning; //!< logging commands will be ignored
  static ContextWarning<> badclose_warning; //!< close request with no open file

private:
  logfile_controller(const logfile_controller&); //!< prevent copy and assignment
  logfile_controller& operator= (const logfile_controller&);

  int flags; //!< log file flags
  matlab_controller* filep_matlab; //!< Matlab file controller (NULL if not Matlab)
  FILE* filep_ascii; //!< ASCII file pointer (NULL if not ASCII)
};

void reset_logcount(); //!< reset counters used to tag log output
void update_log_file(const char* fname, int flags); //!< change log file to \a fname
//extern smartptr<logfile_controller,false> log_filep;
logfile_controller* get_logfile(); //!< pointer to log file controller object (NULL if non active)

#endif
