speed_light_si = 299792458.0
electron_mass_si = 9.10938215e-31
elementary_charge_si = 1.602176487e-19
mu_0_si = 4.0*math.pi*1e-7
epsilon_0_si = 1.0/(mu_0_si*speed_light_si**2)
planck_si = 6.62606896e-34
hbar_si = planck_si / (2.0 * math.pi)
fine_structure_si = elementary_charge_si**2/(4.0*math.pi*epsilon_0_si*hbar_si*speed_light_si)
metre      = electron_mass_si*speed_light_si*fine_structure_si/hbar_si
barn         = metre*metre*1.0e-28
millibarn    = barn*1.0e-3

joule              = 1.0/(fine_structure_si**2*electron_mass_si*speed_light_si**2)
hertz              = planck_si*joule
megahertz          = hertz*1e6

def efg_to_Cq(efg, s):
  # Magic constants to convert from millibarns to a.u. and then back to MHz
  if s in Q_iso:
    return sorted_evals(efg)[0] * (Q_common[s] * millibarn) / megahertz
  else:
    return None

def efg_to_Cq_isotope(efg, species, isotope):
  # Magic constants to convert from millibarns to a.u. and then back to MHz
  return sorted_evals(efg)[0] * (Q[(species, isotope)] * millibarn) / megahertz

def val_to_Cq(ev, s):
  if s in Q_iso:
    return ev * Q_common[s] * millibarn / megahertz
  else:
    return ev

def K_to_J(K, s1, s2):
  # More magic constants. Should make a proper atomic unit conversion function
  return (K + K.T)/2.0 * gamma_common[s1] * gamma_common[s2] * 1.05457148e-15 / (2*math.pi)

def K_to_J_iso(K, s1, iso1, s2, iso2):
  return (K + K.T)/2.0 * gamma[(s1,iso1)] * gamma[(s2,iso2)] * 1.05457148e-15 / (2*math.pi)

# Nuclear gyromagnetic ratios, from constants.f90, source IUPAC Recommendations 2001, Robin K. Harris et al
gamma={('H', 1): 26.7522128e7,
       ('H', 2): 4.10662791e7,
       ('H', 3): 28.5349779e7,
       ('He', 3): -20.3801587e7,
       ('Li', 6): 3.9371709e7,
       ('Li', 7): 10.3977013e7,
       ('Be', 9): -3.759666e7,
       ('B', 10): 2.8746786e7,
       ('B', 11): 8.5847044e7,
       ('C', 13): 6.728284e7,
       ('N', 14): 1.9337792e7,
       ('N', 15): -2.71261804e7,
       ('O', 17): -3.62808e7,
       ('F', 19): 25.18148e7,
       ('Ne', 21): -2.11308e7,
       ('Na', 23): 7.0808493e7,
       ('Mg', 25): -1.63887e7, 
       ('Al', 27): 6.9762715e7,
       ('Si', 29): -5.3190e7,
       ('P', 31): 10.8394e7,
       ('S', 33): 2.055685e7,                  
       ('Cl', 35): 2.624198e7,
       ('Cl', 37): 2.184368e7,
       ('K', 39): 1.2500608e7,
       ('K', 40): -1.5542854e7,
       ('K', 41): 0.68606808e7,
       ('Ca', 43): -1.803069e7,           
       ('Sc', 45): 6.5087973e7,
       ('Ti', 47): -1.5105e7,
       ('Ti', 49): -1.51095e7,
       ('V', 50): 2.6706490e7,
       ('V', 51): 7.0455117e7,
       ('Cr', 53): -1.5152e7,
       ('Mn', 55): 6.6452546e7,
       ('Fe', 57): 0.8680624e7,
       ('Co', 59): 6.332e7,
       ('Ni', 61): -2.3948e7,
       ('Cu', 63): 7.1117890e7,
       ('Cu', 65): 7.60435e7,
       ('Zn', 67): 1.676688e7,
       ('Ga', 69): 6.438855e7,
       ('Ga', 71): 8.181171e7,
       ('Ge', 73): -0.9360303e7,                  
       ('As', 75): 4.596163e7,
       ('Se', 77): 5.1253857e7,
       ('Br', 79): 6.725616e7,
       ('Br', 81): 7.249776e7,
       ('Kr', 83): -1.03310e7,              
       ('Rb', 85): 2.5927050e7,
       ('Rb', 87): 8.786400e7,
       ('Sr', 87): -1.1639376e7,
       ('Y', 89): -1.3162791e7,
       ('Zr', 91): -2.49743e7, 
       ('Nb', 93): 6.5674e7,
       ('Mo', 95): 1.751e7,
       ('Mo', 97): -1.788e7,
       ('Tc', 99): 6.046e7,
       ('Ru', 99): -1.229e7,
       ('Rh', 103): -0.8468e7,
       ('Pd', 105): -1.23e7,
       ('Ag', 107): -1.0889181e7,
       ('Ag', 109): -1.2518634e7,
       ('Cd', 111): -5.6983131e7,
       ('Cd', 113): -5.9609155e7,
       ('In', 113): 5.8845e7,
       ('In', 115): 5.8972e7,
       ('Sn', 115): -8.8013e7,
       ('Sn', 117): -9.58879e7,
       ('Sn', 119): -10.0317e7,
       ('Sb', 121): 6.4435e7,
       ('Sb', 123): 3.4892e7,
       ('Te', 123): -7.059098e7,
       ('Te', 125): -8.5108404e7,                      
       ('I', 127): 5.389573e7,
       ('Xe', 131): 2.209076e7,
       ('Cs', 133): 3.5332539e7,
       ('Ba', 135): 2.67550e7,
       ('Ba', 137): 2.99295e7,
       ('La', 138): 3.557239e7,
       ('La', 139): 3.8083318e7,
       ('Pr', 141): 8.1907e7,
       ('Nd', 143): -1.457e7,
       ('Nd', 145): -0.898e7,
       ('Sm', 147): -1.115e7,
       ('Sm', 149): -0.9192e7,
       ('Eu', 151): 6.6510e7,
       ('Eu', 153): 2.9369e7,
       ('Gd', 155): -0.82132e7,
       ('Gd', 157): -1.0769e7,
       ('Tb', 159): 6.431e7,
       ('Dy', 161): -0.9201e7,
       ('Dy', 163): 1.289e7,
       ('Ho', 165): 5.710e7,
       ('Er', 167): -0.77157e7,
       ('Tm', 169): -2.218e7,
       ('Yb', 171): 4.7288e7,
       ('Yb', 173): -1.3025e7,
       ('Lu', 175): 3.0552e7,
       ('Lu', 176): 2.1684e7,
       ('Hf', 177): 1.086e7,
       ('Hf', 179): -0.6821,
       ('Ta', 181): 3.2438e7,
       ('W', 183): 1.1282403e7,
       ('Re', 185): 6.1057e7,
       ('Re', 187): 6.1682e7,
       ('Os', 189): 2.10713e7,
       ('Ir', 191): 0.4812e7,
       ('Ir', 193): 0.5227e7,
       ('Pt', 195): 5.8385e7,
       ('Au', 197): 0.473060e7,
       ('Hg', 201): -1.788769e7,
       ('Hg', 199): 4.8457916e7,
       ('Tl', 203): 15.5393338e7,
       ('Tl', 205): 15.6921808e7,
       ('Pb', 207): 5.58046e7,
       ('Bi', 209): 4.3750e7,
       ('U', 235): -0.52e7,}

# Isotope of most common spin active nucleus for each species
gamma_iso = {'H': 1,
             'He': 3,
             'Li': 7,
             'Be': 9,
             'B': 11,
             'C': 13,
             'N': 14, #
             'O': 17,
             'F': 19,
             'Ne': 21,
             'Na': 23,
             'Mg': 25,
             'Al': 27,
             'Si': 29,
             'P': 31,
             'S': 33,
             'Cl': 35,
             'K': 39,
             'Ca': 43,
             'Sc': 45,
             'Ti': 49, # Maybe exclude arbitrary ones from this? 47Ti and 49Ti both fairly equal, UI can prompt
             'V': 51,
             'Cr': 53,
             'Mn': 55,
             'Fe': 57,
             'Co': 59,
             'Ni': 61,
             'Cu': 65, #
             'Zn': 67,
             'Ga': 71,
             'Ge': 73,
             'As': 75,
             'Se': 77,
             'Br': 81,
             'Kr': 83,
             'Rb': 87,
             'Sr': 87,
             'Y': 89,
             'Zr': 91,
             'Nb': 93,
             'Mo': 95,
             'Tc': 99,
             'Ru': 99, # Also 101
             'Rh': 103,
             'Pd': 105,
             'Ag': 109, #
             'Cd': 113,
             'In': 115,
             'Sb': 121,
             'I': 127,
             'Sn': 119,
             'Te': 125,
             'Xe': 129, # Also 131
             'Cs': 133,
             'Ba': 137,
             'La': 139,
             'Hf': 179, # Also 177
             'Ta': 181,
             'W': 183,
             'Re': 187,
             'Os': 187, # Also 189 ?? no 187Os in gamma list
             'Ir': 193,
             'Pt': 195,
             'Au': 197,
             'Hg': 199, # Also 201
             'Tl': 205,
             'Pb': 207,
             'Bi': 209,}

gamma_common = {}
for s, i in gamma_iso.items():
  if (s,i) in gamma:
   gamma_common[s] = gamma[(s,i)]

# Quadrupole moments of all nuclear isotopes, from P. Pyykko, J. Mol. Phys, 2008 106 1965-1974
# Units of mb, Q/10/fm^2
Q = {('H', 2): 2.860,
     ('Li', 6): -0.808,
     ('Li', 7): -40.1,
     ('Be', 9): 52.88,
     ('B', 10): 84.59,
     ('B', 11): 40.59,
     ('C', 11): 33.27,
     ('N', 14): 20.44,
     ('O', 17): -25.58,
     ('F', 19): -94.2,
     ('Ne', 21): 101.55,
     ('Na', 23): 104,
     ('Mg', 25): 199.4,
     ('Al', 27): 146.6,
     ('S', 33): -67.8,
     ('S', 35): 47.1,
     ('Cl', 35): -81.65,
     ('Cl', 37): -64.35,
     ('K', 39): 58.5,
     ('K', 40): -73,
     ('K', 41): 71.1,
     ('Ca', 41): -66.5,
     ('Ca', 43): -40.8,
     ('Sc', 45): -220,
     ('Ti', 47): 302,
     ('Ti', 49): 247,
     ('V', 50): 210,
     ('V', 51): -52,
     ('Cr', 53): -150,
     ('Mn', 55): 330,
     ('Fe', 57): 160,
     ('Co', 59): 420,
     ('Ni', 61): 162,
     ('Cu', 63): -220,
     ('Cu', 65): -204,
     ('Zn', 67): 150,
     ('Ga', 69): 171,
     ('Ga', 71): 107,
     ('Ge', 73): -196,
     ('As', 75): 314,
     ('Se', 77): 760,
     ('Br', 79): 313,
     ('Br', 81): 262,
     ('Kr', 83): 259,
     ('Rb', 85): 276,
     ('Rb', 87): 133.5,
     ('Sr', 87): 305,
     ('Y', 90): -125,
     ('Zr', 91): -176,
     ('Nb', 93): -320,
     ('Mo', 95): -22,
     ('Mo', 97): 255,
     ('Tc', 99): -129,
     ('Ru', 99): 79,
     ('Ru', 101): 457,
     ('Pd', 105): 660,
     ('In', 113): 759,
     ('In', 115): 770,
     ('Sn', 119): -132,
     ('Sb', 121): -543,
     ('Sb', 123): -692,
     ('I', 127): -696,
     ('Xe', 131): -114,
     ('Cs', 133): -3.43,
     ('Ba', 135): 160,
     ('Ba', 137): 245,
     ('La', 138): 450,
     ('La', 139): 200,
     ('Pr', 141): -58.9,
     ('Nd', 143): -630,
     ('Nd', 145): -330,
     ('Pm', 147): 740,
     ('Sm', 147): -259,
     ('Sm', 149): 75,
     ('Eu', 151): 903,
     ('Eu', 153): 2412,
     ('Gd', 155): 1270,
     ('Gd', 157): 1350,
     ('Tb', 159): 1432,
     ('Dy', 161): 2507,
     ('Dy', 163): 2648,
     ('Ho', 165): 3580,
     ('Er', 167): 3565,
     ('Tm', 169): -1200, # Tm missing from Pyykko
     ('Yb', 173): 2800,
     ('Lu', 175): 3490,
     ('Lu', 176): 4970,
     ('Hf', 177): 3365,
     ('Hf', 179): 3793,
     ('Ta', 181): 3170,
     ('Re', 185): 2180,
     ('Re', 187): 2070,
     ('Os', 189): 856,
     ('Ir', 191): 816,
     ('Ir', 193): 751,
     ('Au', 197): 547,
     ('Hg', 201): 387,
     ('Pb', 209): -269,
     ('Bi', 209): -516,
     ('Rn', 209): 311,
     ('Fr', 223): 1170,
     ('Ra', 223): 1210, # Ra missing from Pyykko
     ('Ac', 227): 1700,
     ('Th', 229): 4300,
     ('Pa', 231): -1720,
     ('U', 233): 3663,
     ('U', 235): 4936,
     ('Np', 237): 3886,
     ('Pu', 241): 5600,
     ('Am', 243): 4210,
     ('Es', 253): 6700,
    }

# Isotope of most common quadrupolar nucleus, from castep constants.f90
Q_iso = {'H': 2,
         'Li': 7,
         'Be': 9,
         'B': 11,
         'C': 11,
         'N': 14,
         'O': 17,
         'Ne': 21,
         'Na': 23,
         'Mg': 25,
         'Al': 27,
         'S': 33,
         'Cl': 35,
         'K': 39,
         'Ca': 43,
         'Sc': 45,
         'Ti': 47,
         'V': 51,
         'Cr': 53,
         'Mn': 55,
         'Fe': 57,
         'Co': 59,
         'Ni': 61,
         'Cu': 63,
         'Zn': 67,
         'Ga': 71,
         'Ge': 73,
         'As': 75,
         'Br': 81,
         'Kr': 83,
         'Rb': 87,
         'Sr': 87,
         'Y': 90,
         'Zr': 91,
         'Nb': 93,
         'Mo': 95,
         'Tc': 99,
         'Ru': 99,
         'Pd': 105,
         'In': 115,
         'Sb': 121,
         'I': 127,
         'Xe': 131,
         'Cs': 133,
         'Ba': 137,
         'La': 139,
         'Pr': 141,
         'Nd': 143,
         'Pm': 147,
         'Sm': 149,
         'Eu': 153,
         'Gd': 157,
         'Tb': 159,
         'Dy': 163,
         'Ho': 165,
         'Er': 167,
         'Tm': 169,
         'Yb': 173,
         'Lu': 175,
         'Hf': 177,
         'Ta': 181,
         'Re': 187,
         'Os': 189,
         'Ir': 193,
         'Au': 197,
         'Hg': 201,
         'Pb': 209,
         'Bi': 209,
         'Rn': 209,
         'Fr': 223,
         'Ra': 223,
         'Ac': 227,
         'Th': 229,
         'Pa': 231,
         'U': 235,
         'Np': 237,
         'Pu': 241,
         'Am': 243,
         'Es': 253,}

Q_common = {}
for s, i in Q_iso.items():
  if (s,i) in gamma:
    Q_common[s] = Q[(s,i)]

iso_spin = {('Ne', 22): 0.0, ('Cl', 35): 1.5, ('Tl', 205): 0.5, ('Te', 123): 0.5, ('W', 180): 0.0, ('Pm', 147): 3.5, ('Tb', 160): 3.0, ('Cd', 106): 0.0, ('Sb', 121): 2.5, ('Hf', 180): 0.0, ('Ac', 227): 1.5, ('Eu', 151): 2.5, ('Pb', 206): 0.0, ('Kr', 80): 0.0, ('As', 75): 1.5, ('K', 41): 1.5, ('Pb', 204): 0.0, ('Sb', 125): 3.5, ('Zr', 90): 0.0, ('Sn', 117): 0.5, ('In', 113): 4.5, ('Os', 189): 1.5, ('Ca', 48): 0.0, ('Gd', 157): 1.5, ('S', 36): 0.0, ('Ce', 136): 0.0, ('Xe', 124): 0.0, ('Tb', 157): 1.5, ('Hf', 179): 4.5, ('Cs', 133): 3.5, ('Ge', 72): 0.0, ('Nb', 93): 4.5, ('Sn', 115): 0.5, ('Kr', 78): 0.0, ('Mo', 95): 2.5, ('Tb', 159): 1.5, ('Kr', 83): 4.5, ('Ca', 43): 3.5, ('Sm', 150): 0.0, ('Se', 78): 0.0, ('Pd', 108): 0.0, ('Sm', 154): 0.0, ('Ar', 36): 0.0, ('Ba', 132): 0.0, ('Se', 80): 0.0, ('Ru', 101): 2.5, ('Xe', 134): 0.0, ('Cl', 37): 1.5, ('Yb', 172): 0.0, ('C', 12): 0.0, ('Yb', 174): 0.0, ('Zn', 66): 0.0, ('Ta', 180): 0.0, ('Cr', 53): 1.5, ('Ba', 133): 0.5, ('Pt', 195): 0.5, ('Ni', 62): 0.0, ('Rh', 102): 6.0, ('Er', 170): 0.0, ('Er', 167): 3.5, ('Cd', 108): 0.0, ('Fe', 57): 0.5, ('Cu', 65): 1.5, ('Ta', 181): 3.5, ('Ru', 99): 2.5, ('Fe', 54): 0.0, ('Cd', 112): 0.0, ('B', 10): 3.0, ('Lu', 176): 7.0, ('Ca', 44): 0.0, ('Y', 89): 0.5, ('Te', 120): 0.0, ('Pd', 105): 2.5, ('Sb', 123): 3.5, ('Xe', 136): 0.0, ('Be', 9): 1.5, ('Mo', 97): 2.5, ('He', 3): 0.5, ('Ru', 96): 0.0, ('Gd', 154): 0.0, ('Rb', 85): 2.5, ('Zr', 91): 2.5, ('Re', 185): 2.5, ('Ba', 138): 0.0, ('Yb', 171): 0.5, ('Th', 232): 0.0, ('Hf', 176): 0.0, ('Dy', 156): 0.0, ('Sn', 120): 0.0, ('H', 3): 0.5, ('Na', 22): 3.0, ('Dy', 158): 0.0, ('Ga', 69): 1.5, ('Sm', 144): 0.0, ('Pb', 208): 0.0, ('Si', 28): 0.0, ('Pt', 196): 0.0, ('Zr', 94): 0.0, ('Mg', 24): 0.0, ('Pd', 106): 0.0, ('Ne', 21): 1.5, ('Ge', 76): 0.0, ('S', 32): 0.0, ('Te', 122): 0.0, ('Se', 82): 0.0, ('Xe', 130): 0.0, ('Dy', 164): 0.0, ('W', 184): 0.0, ('Hg', 201): 1.5, ('Cd', 114): 0.0, ('Os', 190): 0.0, ('Eu', 152): 3.0, ('Po', 209): 0.5, ('Se', 77): 0.5, ('Ag', 109): 0.5, ('Dy', 160): 0.0, ('Hg', 198): 0.0, ('Tm', 171): 0.5, ('Kr', 84): 0.0, ('Ar', 40): 0.0, ('Sn', 116): 0.0, ('Co', 60): 5.0, ('Nd', 142): 0.0, ('Sr', 84): 0.0, ('W', 182): 0.0, ('Gd', 155): 1.5, ('Ce', 140): 0.0, ('Ir', 193): 1.5, ('Ge', 73): 4.5, ('Tc', 99): 4.5, ('Gd', 160): 0.0, ('Hf', 177): 3.5, ('Tl', 204): 2.0, ('Mo', 92): 0.0, ('Gd', 158): 0.0, ('Kr', 82): 0.0, ('Ca', 40): 0.0, ('Se', 79): 3.5, ('I', 129): 3.5, ('Ar', 39): 3.5, ('Sn', 119): 0.5, ('Pr', 141): 2.5, ('Ru', 100): 0.0, ('Sr', 87): 4.5, ('Cl', 36): 2.0, ('Mn', 55): 2.5, ('C', 13): 0.5, ('Pt', 194): 0.0, ('Zn', 67): 2.5, ('Sm', 149): 3.5, ('Cr', 52): 0.0, ('Xe', 131): 1.5, ('Yb', 168): 0.0, ('Ni', 61): 1.5, ('Rh', 103): 0.5, ('N', 15): 0.5, ('Fe', 56): 0.0, ('Ge', 74): 0.0, ('Yb', 176): 0.0, ('Er', 166): 0.0, ('Bi', 207): 4.5, ('Se', 76): 0.0, ('Ag', 107): 0.5, ('Pt', 192): 0.0, ('Co', 59): 3.5, ('Pd', 110): 0.0, ('Dy', 163): 2.5, ('Hg', 204): 0.0, ('Cu', 63): 1.5, ('Sm', 148): 0.0, ('Te', 126): 0.0, ('Nd', 144): 0.0, ('Xe', 132): 0.0, ('La', 139): 3.5, ('Ni', 64): 0.0, ('C', 14): 3.0, ('Zn', 68): 0.0, ('Ti', 50): 0.0, ('Cd', 113): 0.5, ('Ba', 137): 1.5, ('Yb', 170): 0.0, ('Ce', 138): 0.0, ('Er', 168): 0.0, ('O', 16): 0.0, ('Lu', 175): 3.5, ('H', 2): 1.0, ('Br', 81): 1.5, ('Pt', 190): 0.0, ('Gd', 152): 0.0, ('Sn', 118): 0.0, ('Si', 29): 0.5, ('Ca', 46): 0.0, ('Sc', 45): 3.5, ('Ho', 165): 3.5, ('Mg', 25): 2.5, ('Os', 186): 0.0, ('Ne', 20): 0.0, ('Bi', 209): 4.5, ('Tl', 203): 0.5, ('Eu', 155): 2.5, ('S', 33): 1.5, ('Te', 125): 0.5, ('Ru', 98): 0.0, ('V', 51): 3.5, ('Ba', 135): 1.5, ('Rb', 87): 1.5, ('Sn', 112): 0.0, ('Ti', 49): 3.5, ('Cr', 50): 0.0, ('Xe', 128): 0.0, ('Cd', 111): 0.5, ('Nd', 148): 0.0, ('Th', 229): 2.5, ('U', 238): 0.0, ('Eu', 153): 2.5, ('Ga', 71): 1.5, ('Ti', 47): 2.5, ('Cs', 137): 3.5, ('Sn', 122): 0.0, ('Si', 30): 0.0, ('Sr', 88): 0.0, ('Mg', 26): 0.0, ('In', 115): 4.5, ('Nd', 143): 3.5, ('W', 183): 0.5, ('Mo', 100): 0.0, ('Hg', 199): 0.5, ('S', 34): 0.0, ('He', 4): 0.0, ('Ir', 191): 1.5, ('Xe', 126): 0.0, ('W', 186): 0.0, ('Re', 187): 2.5, ('F', 19): 0.5, ('Ge', 70): 0.0, ('Er', 162): 0.0, ('Os', 192): 0.0, ('Au', 197): 1.5, ('Te', 130): 0.0, ('U', 234): 0.0, ('Ca', 41): 3.5, ('Hf', 174): 0.0, ('K', 40): 4.0, ('Tm', 169): 0.5, ('Sm', 152): 0.0, ('Ar', 38): 0.0, ('K', 39): 1.5, ('Os', 188): 0.0, ('Sr', 86): 0.0, ('U', 235): 3.5, ('P', 31): 0.5, ('Zn', 64): 0.0, ('Ru', 104): 0.0, ('Ni', 60): 0.0, ('Ce', 142): 0.0, ('Sm', 151): 2.5, ('N', 14): 1.0, ('Xe', 129): 0.5, ('Ba', 130): 0.0, ('Cd', 110): 0.0, ('Mo', 94): 0.0, ('Gd', 156): 0.0, ('Hg', 200): 0.0, ('Ca', 42): 0.0, ('Sn', 124): 0.0, ('Pt', 198): 0.0, ('Br', 79): 1.5, ('Li', 6): 1.0, ('Dy', 162): 0.0, ('Cd', 116): 0.0, ('Ru', 102): 0.0, ('Cs', 134): 4.0, ('La', 138): 5.0, ('Mn', 53): 3.5, ('Mo', 98): 0.0, ('Cr', 54): 0.0, ('Ba', 136): 0.0, ('Yb', 173): 2.5, ('Hf', 178): 0.0, ('O', 17): 2.5, ('I', 127): 2.5, ('Fe', 58): 0.0, ('H', 1): 0.5, ('Os', 184): 0.0, ('Er', 164): 0.0, ('Hg', 202): 0.0, ('Lu', 173): 3.5, ('B', 11): 1.5, ('Lu', 174): 1.0, ('Al', 27): 2.5, ('Sm', 147): 3.5, ('Zr', 92): 0.0, ('Se', 74): 0.0, ('Pd', 104): 0.0, ('Os', 187): 0.5, ('Dy', 161): 2.5, ('Mo', 96): 0.0, ('Te', 124): 0.0, ('Nd', 150): 0.0, ('V', 50): 6.0, ('La', 137): 3.5, ('Te', 128): 0.0, ('Zn', 70): 0.0, ('Ti', 48): 0.0, ('Zr', 96): 0.0, ('Nd', 146): 0.0, ('Ni', 58): 0.0, ('Cs', 135): 3.5, ('O', 18): 0.0, ('Eu', 154): 3.0, ('Na', 23): 1.5, ('Kr', 85): 4.5, ('Ti', 46): 0.0, ('Ba', 134): 0.0, ('Pb', 207): 0.5, ('Pd', 102): 0.0, ('Kr', 86): 0.0, ('Hg', 196): 0.0, ('Nd', 145): 3.5, ('Sn', 114): 0.0, ('Li', 7): 1.5}

